<?php

define('DHL_PROTOCOL', 'https://');
define('DHL_DOMAIN', 'dhl24.com.pl');
define('FAILURE', 'false');

define('DHL_MAGENTOAPI', 'https://dhl24.com.pl/magentoapi');
define('DHL_WEBAPI2', 'https://dhl24.com.pl/webapi2');
define('DHL_SERVICEPOINTAPI', 'https://dhl24.com.pl/servicepoint');

class DHL_Dhl24pl_Adminhtml_IndexController extends Mage_Adminhtml_Controller_Action
{
 	private $section;
 	private $sid = '';
 	private $pid;
 	private $oid;

    public function createAction() {
        $this->loadLayout();
        $shipmentblock = Mage::app()->getLayout()->getBlock('shipment_shipmentbloc');

        $oid = $this->getRequest()->getParam('oid');
        if (!$this->isOrder($oid)) {
            Mage::getSingleton('core/session')->addError('Niepoprawne zamówienie');
            $this->_redirect('adminhtml/sales_order/index');
            return;
        }
        if ($this->isAlreadyCreated($oid)) {
            Mage::getSingleton('core/session')->addError('Dla wybranego zamówienia nie można utworzyć przesyłki');
            $this->_redirect('adminhtml/sales_order/index');
            return;
        }
        //generujemy dni do formularza zamawiania kuriera
        $shipmentblock->generateDays();
        $magentoData = $this->getMagentoFromDHL();
        if ($magentoData['error'] !== false) {
            Mage::getSingleton('core/session')->addError($magentoData['error']);
            $this->_redirect('adminhtml/sales_order/index');
            return;
        }
        $magentoData = $magentoData['data'];

//        //ustawiamy informacje czy dostepne LM
        $magentoData['lm'] = $this->isLm();

        //pobieramy domyslna przesylke
        $shipment = array();
        if (isset($magentoData['shipments']) && is_array($magentoData['shipments']) && count($magentoData['shipments']) > 0) {
            foreach ($magentoData['shipments'] as $key => $value) {
                $shipment = $value;
                break;
            }
        }
        //sprawdzamy czy mozemy nadac kuriera
        $shipmentName = '';
        if (isset($shipment['name'])) {
            $shipmentName = $shipment['name'];
        }
        if ($this->canCourier($magentoData, $shipmentName)) {
            $magentoData['courier'] = true;
        } else {
            $magentoData['courier'] = false;
        }
        //pobieramy nadawce
        $sender = array();
        if (isset($shipment['senderName']) && isset($magentoData['senders'][$shipment['senderName']])) {
            $sender = $magentoData['senders'][$shipment['senderName']];
        } else {
            foreach ($magentoData['senders'] as $key => $value) {
                $sender = $value;
                break;
            }
        }
        //zmianna dla platnik 3 strona
        $sender['name2'] = '';

        //pobieramy dane odbiorcy
        $reciver = $this->getFromOrderData($oid);

        //pobieranie domyslnej paczki
        $shipment['Package'] = array();
        if (isset($shipment['packageType'])) {
            $package = array();
            $package['packageType'] = $shipment['packageType'];
            $package['width'] = $shipment['width'];
            $package['height'] = $shipment['height'];
            $package['length'] = $shipment['length'];
            $package['weight'] = $shipment['weight'];
            $package['quantity'] = $shipment['quantity'];
            $package['nonStandard'] = $shipment['nonStandard'];
            $shipment['Package'][] = $package;
        }
        $shipment['comment'] = $oid;
        $shipment['shipmentDate'] = date('Y-m-d');
        //ustawiamy domyslnego platnika
        if (!isset($shipment['payer']) || empty($shipment['payer'])) {
            $shipment['payer'] = 'N';
        }

        //tworzymy tablice z wszytskimi danymi
        $allShipment = array_merge($shipment, array_merge($reciver, $sender));
        if (isset($_POST["order_id"])) {
            $allShipment = $_POST;

            //dane nadawcy z panelu magento - do sekcji billing
            $magentoSender = false;
            if (isset($_POST['name'])) {
                if (isset($magentoData['senders'][$_POST['name']])) {
                    $magentoSender = $magentoData['senders'][$_POST['name']];
                }
            }
            if ($magentoSender === false) {
                foreach ($magentoData['senders'] as $key => $value) {
                    $magentoSender = $value;
                    break;
                }
            }
            //pobieramy dane nadawcy gdy na formularzy jest disabled, nie ma w $_POST
            $result = array();
            if (isset($_POST['payer']) && ($_POST['payer'] == 'N' || $_POST['payer'] == 'O')) {
                $name = $_POST['name'];
                if (isset($magentoData['senders'][$name])) {
                    $result = $magentoData['senders'][$name];
                }
            }

            //ustawimy czy mozna zamowic kuriera
            $shipmentName = '';
            if (isset($allShipment['shipment'])) {
                $shipmentName = $allShipment['shipment'];
            }
            if ($this->canCourier($magentoData, $shipmentName)) {
                $magentoData['courier'] = true;
            } else {
                $magentoData['courier'] = false;
            }

            $allShipment = array_merge($allShipment, $result);
            $shipmentblock->setShipmentValues($allShipment);
            if ($shipmentblock->validate()) {
                try {
                    if ($this->isLm() && $this->isLmShipment($allShipment)) {
                        $result = $this->createServicePointShipment($allShipment, $magentoData, $magentoSender);
                        $order = Mage::getModel('sales/order')->loadByIncrementId($oid);
                        $params = array(
                            'lp' => $result->createShipmentResult->shipmentNumber,
                            'labeltype' => $result->createShipmentResult->label->labelType,
                            'type' => 'SERVICEPOINT'
                        );
                        $order->setDhl24plSettings(json_encode($params));
                        $order->save();

                        $this->getResponse ()
                            ->setHeader ( 'Content-type', $result->createShipmentResult->label->labelFormat )
                            ->setHeader ('Content-Disposition', 'attachment' . '; filename=' . $result->createShipmentResult->label->labelName );
                        $this->getResponse ()->setBody(base64_decode($result->createShipmentResult->label->labelContent));
                    } else {
                        $result = $this->createWebapiShipment($allShipment, $magentoData, $magentoSender);
                        $order = Mage::getModel('sales/order')->loadByIncrementId($oid);
                        $params = array(
                            'lp' => $result->createShipmentResult->shipmentNotificationNumber,
                            'labeltype' => $result->createShipmentResult->label->labelType,
                            'type' => 'WEBAPI'
                        );
                        $order->setDhl24plSettings(json_encode($params));
                        $order->save();
                        $name = $result->createShipmentResult->shipmentNotificationNumber . '.pdf';
                        if ($result->createShipmentResult->label->labelType == 'ZBLP') {
                            $name = $result->createShipmentResult->shipmentNotificationNumber . '.zpl';
                        }

                        $this->getResponse ()
                            ->setHeader ( 'Content-type', $result->createShipmentResult->label->labelFormat )
                            ->setHeader ('Content-Disposition', 'attachment' . '; filename=' . $name );
                        $this->getResponse ()->setBody(base64_decode($result->createShipmentResult->label->labelContent));
                    }
                } catch (Exception $e) {
                    $shipmentblock->setError($e->getMessage());
                }
            }
        }
        $shipmentblock->setMagentoData($magentoData);
        $shipmentblock->setShipmentValues($allShipment);
        $shipmentblock->setOrderId($oid);

        $head = Mage::app()->getLayout()->getBlock('head');
        $head->addItem('js_css', 'css/dhl24pl.css');
        $this->renderLayout();
    }

    /** Metoda czyści nr telefonu tak aby były same cyfry
     * @param $number
     * @return mixed
     */
    private function clearPhone($number) {
        $number = str_replace(array(' ', '+', '-'), array('', '', ''), $number);
        return $number;
    }

    /**
     * Akcja wyświetlająca formularz generowania raportu PNP oraz samo generowanie raportu
     */
    public function pnpAction() {
        $this->loadLayout();
        $pnpblock = Mage::app()->getLayout()->getBlock('pnp_pnpbloc');
        if (isset($_POST['date'])) {
            if (isset($_POST['kind'])) {
                $pnpblock->setKind($_POST['kind']);
            }
            if (isset($_POST['date'])) {
                $pnpblock->setDate($_POST['date']);
            }
            if (isset($_POST['type'])) {
                $pnpblock->setType($_POST['type']);
            }
            if ( $pnpblock->validate()) {
                try {
                    if ($_POST['kind'] == 'WEBAPI') {
                        $result = $this->generateWebapiPnp($_POST['date'], $_POST['type']);
                        header('Content-type: ' . $result->getPnpResult->fileMimeType);
                        header('Content-Disposition: attachment; filename="' . $result->getPnpResult->fileName . '"');
                        echo base64_decode($result->getPnpResult->fileData);
                    } else {
                        $result = $this->generateServicepointPnp($_POST['date']);
                        header('Content-type: ' . $result->getPnpResult->labelFormat);
                        header('Content-Disposition: attachment; filename="' . $result->getPnpResult->labelName . '"');
                        echo base64_decode($result->getPnpResult->labelContent);
                    }
                } catch (Exception $e) {
                    $error = $e->getMessage();
                }
            }
        }
        $head = Mage::app()->getLayout()->getBlock('head');
        $head->addItem('js_css', 'css/dhl24pl.css');
        $this->renderLayout();
    }

    /**
     * Akcja do usuwania utworzonej przesyłki
     */
    public function removeAction() {
        $oid = $this->getRequest()->getParam('oid');
        $order = $this->getOrder($oid);
        if ($order === false) {
            Mage::getSingleton('core/session')->addError('Nie udało się znaleść zamówienia');
            $this->_redirect('adminhtml/sales_order/index');
            return;
        }
        $settings = $order->getDhl24plSettings();
        if (empty($settings)) {
            Mage::getSingleton('core/session')->addError('Do zamówienia nie jest przypisana przesyłka');
            $this->_redirect('adminhtml/sales_order/index');
            return;
        }
        $settings = json_decode($settings);

        if (isset($settings->pid)) {
            Mage::getSingleton('core/session')->addError('Przesyłka została utworzona starą metodą, usuwanie nie wspierane dla takiej przesyłki');
            $this->_redirect('adminhtml/sales_order/index');
            return;
        }
        if (!isset($settings->lp) || empty($settings->lp)) {
            Mage::getSingleton('core/session')->addError('Zamówienie nie posiada przypisanego nuemru zamówienia');
            $this->_redirect('adminhtml/sales_order/index');
            return;
        }
        if (isset($settings->type) && $settings->type == 'SERVICEPOINT') {//service pont
            if (!$this->isLm()) {
                Mage::getSingleton('core/session')->addError('Nie można usunąć przesyłki. Brak danych dostępowych do api servicepoint');
                $this->_redirect('adminhtml/sales_order/index');
                return;
            }
            try{
                $result = $this->removeServicePointShipment($settings->lp);
                if ($result->deleteShipmentResult->status == 'OK') {
                    $order->setDhl24plSettings(0);
                    $order->save();
                    Mage::getSingleton('core/session')->addSuccess('Przesyłka została usunięta.');
                    $this->_redirect('adminhtml/sales_order/index');
                    return;
                } else {
                    Mage::getSingleton('core/session')->addError('Nie można usunąć przesyłki.');
                    $this->_redirect('adminhtml/sales_order/index');
                    return;
                }
            } catch (Exception $e) {
                Mage::getSingleton('core/session')->addError($e->getMessage());
                $this->_redirect('adminhtml/sales_order/index');
                return;
            }
        } else {//webapi
            try {
                $result = $this->removeWebapiShipment($settings->lp);
                if (!$result->deleteShipmentResult->result) {//nie udalo sie usunac przesylki
                    Mage::getSingleton('core/session')->addError('Nie można usunąć przesyłki. ' . $result->deleteShipmentResult->error);
                    $this->_redirect('adminhtml/sales_order/index');
                    return;
                } else {
                    $order->setDhl24plSettings(0);
                    $order->save();
                    Mage::getSingleton('core/session')->addSuccess('Przesyłka została usunięta.');
                    $this->_redirect('adminhtml/sales_order/index');
                    return;
                }
            } catch (Exception $e) {
                Mage::getSingleton('core/session')->addError($e->getMessage());
                $this->_redirect('adminhtml/sales_order/index');
                return;
            }
        }
    }

    /** Akcja do wyświetlania mapy z paczkomatami w Niemczech
     *
     */
    public function showpackstationmapAction() {
        $this->loadLayout();
        $this->renderLayout();
    }

    /**
     * Akcja do wyświetlania mapy service ponts
     */
    public function showlmmapAction() {
        $this->loadLayout();
        $this->renderLayout();
    }

    /**
     * Akcja pobierajaca dane odbiorcy
     */
    public function recipientajaxAction(){
        $oid = $this->getRequest()->getParam('oid');
        $recipient = $this->getFromOrderData($oid);
        $jsonData = json_encode($recipient);
        $this->getResponse()->setHeader('Content-type', 'application/json');
        $this->getResponse()->setBody($jsonData);
    }

    /**
     * Akcja pobierająca dane nadawcy
     */
    public function senderajaxAction(){
        $name = $this->getRequest()->getParam('name');
        $magentoData = $this->getMagentoFromDHL();
        $result = array();
        if ($magentoData['error'] === false) {
            if (isset($magentoData['data']['senders'][$name])) {
                $result = $magentoData['data']['senders'][$name];
            }
        }
        $jsonData = json_encode($result);
        $this->getResponse()->setHeader('Content-type', 'application/json');
        $this->getResponse()->setBody($jsonData);
    }

    /**
     * Akcja pobierająca dane przesyłki
     */
    public function shipmentajaxAction(){
        $name = $this->getRequest()->getParam('name');
        $magentoData = $this->getMagentoFromDHL();
        $result = array();
        if ($magentoData['error'] === false) {
            $magentoData = $magentoData['data'];
            if (isset($magentoData['shipments'][$name])) {
                $result['shipment'] = $magentoData['shipments'][$name];
                $senderName = $magentoData['shipments'][$name]['senderName'];
                if (isset($magentoData['senders'][$senderName])) {
                    $result['sender'] = $magentoData['senders'][$senderName];
                }
            }
            if ($this->canCourier($magentoData, $name)) {
                $result['shipment']['courier'] = true;
            } else {
                $result['shipment']['courier'] = false;
            }
        }
        $jsonData = json_encode($result);
        $this->getResponse()->setHeader('Content-type', 'application/json');
        $this->getResponse()->setBody($jsonData);
    }

    /**
     * Akcja pobierająca dane na temat godizn dostępności kuriera
     */
    public function hoursajaxAction(){
        $date = $this->getRequest()->getParam('date');
        $code = $this->getRequest()->getParam('code');
        try {
            $result = $this->getPostalCodeServices($date, $code);
        } catch (Exception $e) {
            $result = 'Błąd podczas pobierania dostepnych godzin';
        }

        $jsonData = json_encode($result);
        $this->getResponse()->setHeader('Content-type', 'application/json');
        $this->getResponse()->setBody($jsonData);
    }

    /***** METODY API *****/

    /** Metoda generuje etykiete za pomoca api webapi2
     * @param $date data raportu
     * @param $type rodzaj raportu
     * @return mixed
     * @throws CException
     */
    private function generateWebapiPnp($date, $type) {
        $_instance = new SoapClient(
            DHL_WEBAPI2,
            array( 'keep-alive' => true, 'connection_timeout' => 3 )
        );
        if( !$_instance ) {
            throw new CException( 'Brak połączenia z usługą' );
        }
        $params = array(
            'pnpRequest' => array(
                'authData' => array('username' => Mage::getStoreConfig('dhl24pl/webapi/wlogin',0), 'password' => Mage::getStoreConfig('dhl24pl/webapi/wpassword',0)),
                'date' => $date,
                'type' => $type
            )
        );
        $result = $_instance->getPnp( $params );
        return $result;
    }

    /** Metoda generuje etykiete za pomoca api servicepoint
     * @param $date data raportu
     * @return mixed
     * @throws CException
     */
    private function generateServicepointPnp($date) {
        $_instance = new SoapClient(
            DHL_SERVICEPOINTAPI,
            array( 'keep-alive' => true, 'connection_timeout' => 3 )
        );
        if( !$_instance ) {
            throw new CException( 'Brak połączenia z usługą' );
        }
        $params = array(
            'structure' => array(
                'authData' => array('username' => Mage::getStoreConfig('dhl24pl/servicepoint/slogin',0), 'password' => Mage::getStoreConfig('dhl24pl/servicepoint/spassword',0)),
                'shipmentDate' => $date,
            )
        );
        $result = $_instance->getPnp( $params );
        return $result;
    }

    /** Metoda do usuwania przesylki za pomoca api service point
     * @param $lp numer listu przewozowego
     * @return mixed
     * @throws CException
     */
    private function removeServicePointShipment($lp) {
        $_instance = new SoapClient(
            DHL_SERVICEPOINTAPI,
            array( 'keep-alive' => true, 'connection_timeout' => 3 )
        );
        if( !$_instance ) {
            throw new CException( 'Brak połączenia z usługą' );
        }
        $params = array(
            'shipment' => array(
                'authData' => array('username' => Mage::getStoreConfig('dhl24pl/servicepoint/slogin',0), 'password' => Mage::getStoreConfig('dhl24pl/servicepoint/spassword',0)),
                'shipment' => $lp
            )
        );
        $result = $_instance->deleteShipment( $params );
        return $result;
    }

    /** Metoda do usuwania przesylki za pomoca api webapi2
     * @param $lp numer listu przewozowego
     * @return mixed
     * @throws CException
     */
    private function removeWebapiShipment($lp) {
        $_instance = new SoapClient(
            DHL_WEBAPI2,
            array( 'keep-alive' => true, 'connection_timeout' => 3 )
        );
        if( !$_instance ) {
            throw new CException( 'Brak połączenia z usługą' );
        }
        $params = array(
            'authData' => array('username' => Mage::getStoreConfig('dhl24pl/webapi/wlogin',0), 'password' => Mage::getStoreConfig('dhl24pl/webapi/wpassword',0)),
            'shipment' => array(
                'shipmentIdentificationNumber' => $lp
            )
        );
        $result = $_instance->deleteShipment( $params );
        return $result;
    }

    /** Metoda przygotowuje dane wejściowe oraz tworzy przesylke za pomoca metody api webapi2
     * @param $post - dane z formularza
     * @param $magentoData - dane z panelu magento z dhl
     * @param $magentoSender - wybrany nadawca
     * @return mixed
     * @throws CException
     */
    private function createWebapiShipment($post, $magentoData, $magentoSender) {
        $shipment = array();
        $shipmentInfo = array();
        $shipmentInfo['dropOffType'] = $magentoData['dropOffType'];
        if (isset($post['product'])) {
            $shipmentInfo['serviceType'] = $post['product'];
        }
        $billing = array();
        if (isset($post['payer'])) {
            if ($post['payer'] == 'N') {
                $billing['shippingPaymentType'] = 'SHIPPER';
                $billing['paymentType'] = 'BANK_TRANSFER';
            } elseif ($post['payer'] == 'O') {
                $billing['shippingPaymentType'] = 'RECEIVER';
                $billing['paymentType'] = 'CASH';
            } elseif($post['payer'] == 'T') {
                $billing['shippingPaymentType'] = 'USER';
                $billing['paymentType'] = 'BANK_TRANSFER';
            }
        }
        if (isset($magentoSender['sap'])) {
            $billing['billingAccountNumber'] = $magentoSender['sap'];
        }
        if (isset($post['costsCenter'])) {
            $billing['costsCenter'] = $post['costsCenter'];
        }
        $shipmentInfo['billing'] = $billing;
        $shipmentInfo['labelType'] = $magentoData['labelType'];

        $specialServices = array();
        if (isset($post['insurance']) && $post['insurance'] == '1') {
            $item = array();
            $item['serviceType'] = 'UBEZP';
            if (isset($post['insuranceValue'])) {
                $item['serviceValue'] = $post['insuranceValue'];
            }
            $specialServices[] = $item;
        }
        if (isset($post['collectOnDelivery']) && $post['collectOnDelivery'] == '1') {
            $item = array();
            $item['serviceType'] = 'COD';
            if (isset($post['collectOnDeliveryValue'])) {
                $item['serviceValue'] = $post['collectOnDeliveryValue'];
            }
            $item['collectOnDeliveryForm'] = 'BANK_TRANSFER';
            $specialServices[] = $item;
        }
        if (isset($post['predeliveryInformation']) && $post['predeliveryInformation'] == '1') {
            $item = array();
            $item['serviceType'] = 'PDI';
            $specialServices[] = $item;
        }
        if (isset($post['returnOnDelivery']) && $post['returnOnDelivery'] == '1') {
            $item = array();
            $item['serviceType'] = 'ROD';
            $specialServices[] = $item;
        }
        if (isset($post['proofOfDelivery']) && $post['proofOfDelivery'] == '1') {
            $item = array();
            $item['serviceType'] = 'POD';
            $specialServices[] = $item;
        }
        if (isset($post['deliveryEvening']) && $post['deliveryEvening'] == '1') {
            $item = array();
            $item['serviceType'] = '1722';
            $specialServices[] = $item;
        }
        if (isset($post['deliveryOnSaturday']) && $post['deliveryOnSaturday'] == '1') {
            $item = array();
            $item['serviceType'] = 'SOBOTA';
            $specialServices[] = $item;
        }
        if (isset($post['pickupOnSaturday']) && $post['pickupOnSaturday'] == '1') {
            $item = array();
            $item['serviceType'] = 'NAD_SOBOTA';
            $specialServices[] = $item;
        }
        if (isset($post['selfCollect']) && $post['selfCollect'] == '1') {
            $item = array();
            $item['serviceType'] = 'ODB';
            $specialServices[] = $item;
        }
        if (isset($post['deliveryToNeighbour']) && $post['deliveryToNeighbour'] == '1') {
            $item = array();
            $item['serviceType'] = 'SAS';
            $specialServices[] = $item;
        }
        $shipmentInfo['specialServices'] = $specialServices;
        $shipmentTime = array();
        if (isset($post['shipmentDate'])) {
            $shipmentTime['shipmentDate'] = $post['shipmentDate'];
        }
        if (isset($post['shipmentStartHour'])) {
            $shipmentTime['shipmentStartHour'] = $post['shipmentStartHour'];
        }
        if (isset($post['shipmentEndHour'])) {
            $shipmentTime['shipmentEndHour'] = $post['shipmentEndHour'];
        }
        $shipmentInfo['shipmentTime'] = $shipmentTime;
        $shipment['shipmentInfo'] = $shipmentInfo;
        if (isset($post['content'])) {
            $shipment['content'] = $post['content'];
        }
        if (isset($post['comment'])) {
            $shipment['comment'] = $post['comment'];
        }
        if (isset($post['reference'])) {
            $shipment['reference'] = $post['reference'];
        }
        if (isset($post['reference'])) {
            $shipment['reference'] = $post['reference'];
        }

        $ship = array();
        $shipper = array();
        $shipperPreaviso = array();
        $shipperContact = array();
        if (isset($post['personName'])) {
            $shipperPreaviso['personName'] = $post['personName'];
            $shipperContact['personName'] = $post['personName'];

        }
        if (isset($post['phoneNumber'])) {
            $shipperPreaviso['phoneNumber'] = $this->clearPhone($post['phoneNumber']);
            $shipperContact['phoneNumber'] = $this->clearPhone($post['phoneNumber']);

        }
        if (isset($post['emailAddress'])) {
            $shipperPreaviso['emailAddress'] = $post['emailAddress'];
            $shipperContact['emailAddress'] = $post['emailAddress'];
        }
        $shipper['preaviso'] = $shipperPreaviso;
        $shipper['contact'] = $shipperContact;
        $shipperAddress = array();
        if (isset($post['payer'])) {
            if (($post['payer'] == 'N' || $post['payer'] == 'O')) {
                if (isset($post['name'])) {
                    $shipperAddress['name'] = $post['name'];
                }
            } else {
                if (isset($post['name2'])) {
                    $shipperAddress['name'] = $post['name2'];
                }
            }
        }
        if (isset($post['postcode'])) {
            $shipperAddress['postalCode'] = $this->cleanPostalCode($post['postcode']);
        }
        if (isset($post['city'])) {
            $shipperAddress['city'] = $post['city'];
        }
        if (isset($post['street'])) {
            $shipperAddress['street'] = $post['street'];
        }
        if (isset($post['houseNumber'])) {
            $shipperAddress['houseNumber'] = $post['houseNumber'];
        }
        if (isset($post['apartmentNumber'])) {
            $shipperAddress['apartmentNumber'] = $post['apartmentNumber'];
        }
        $shipper['address'] = $shipperAddress;
        $ship['shipper'] = $shipper;
        $receiver = array();
        $receiverPreaviso = array();
        $receiverContact = array();
        if (isset($post['rec_personName'])) {
            $receiverPreaviso['personName'] = $post['rec_personName'];
            $receiverContact['personName'] = $post['rec_personName'];

        }
        if (isset($post['rec_phoneNumber'])) {
            $receiverPreaviso['phoneNumber'] = $this->clearPhone($post['rec_phoneNumber']);
            $receiverContact['phoneNumber'] = $this->clearPhone($post['rec_phoneNumber']);

        }
        if (isset($post['rec_emailAddress'])) {
            $receiverPreaviso['emailAddress'] = $post['rec_emailAddress'];
            $receiverContact['emailAddress'] = $post['rec_emailAddress'];
        }
        $receiver['preaviso'] = $receiverPreaviso;
        $receiver['contact'] = $receiverContact;
        $receiverAddress = array();

        if (isset($post['rec_name'])) {
            $receiverAddress['name'] = $post['rec_name'];
        }
        if (isset($post['rec_postcode'])) {
            $receiverAddress['postalCode'] = $this->cleanPostalCode($post['rec_postcode']);
        }
        if (isset($post['rec_city'])) {
            $receiverAddress['city'] = $post['rec_city'];
        }
        if (isset($post['rec_street'])) {
            $receiverAddress['street'] = $post['rec_street'];
        }
        if (isset($post['rec_houseNumber'])) {
            $receiverAddress['houseNumber'] = $post['rec_houseNumber'];
        }
        if (isset($post['rec_apartmentNumber'])) {
            $receiverAddress['apartmentNumber'] = $post['rec_apartmentNumber'];
        }
        //dodanie danych dla EK
        if (isset($post['product']) && $post['product'] == 'EK') {
            if (isset($post['ek_country'])) {
                $receiverAddress['country'] = $post['ek_country'];
            }
            if (isset($post['ek_packstation']) && !empty($post['ek_packstation'])) {
                $receiverAddress['houseNumber'] = $post['ek_packstation'];
                $receiverAddress['isPackstation'] = true;
            } else {
                $receiverAddress['isPackstation'] = false;
            }
            $receiverAddress['isPostfiliale'] = false;
            if (isset($post['ek_postnummer'])) {
                $receiverAddress['postnummer'] = $post['ek_postnummer'];
            }

        } else {
            $receiverAddress['country'] = 'PL';
        }
        $receiver['address'] = $receiverAddress;
        $ship['receiver'] = $receiver;
        $neighbour = array();
        if (isset($post['deliveryToNeighbour']) && $post['deliveryToNeighbour'] == '1') {
            if (isset($post['neighbour_name'])) {
                $neighbour['name'] = $post['neighbour_name'];
            }
            if (isset($post['neighbour_postcode'])) {
                $neighbour['postalCode'] = $this->cleanPostalCode($post['neighbour_postcode']);
            }
            if (isset($post['neighbour_city'])) {
                $neighbour['city'] = $post['neighbour_city'];
            }
            if (isset($post['neighbour_street'])) {
                $neighbour['street'] = $post['neighbour_street'];
            }
            if (isset($post['neighbour_houseNumber'])) {
                $neighbour['houseNumber'] = $post['neighbour_houseNumber'];
            }
            if (isset($post['neighbour_apartmentNumber'])) {
                $neighbour['apartmentNumber'] = $post['neighbour_apartmentNumber'];
            }
            if (isset($post['neighbour_phoneNumber'])) {
                $neighbour['contactPhone'] = $post['neighbour_phoneNumber'];
            }
            if (isset($post['neighbour_emailAddress'])) {
                $neighbour['contactEmail'] = $post['neighbour_emailAddress'];
            }
        }
        $ship['neighbour'] = $neighbour;
        $shipment['ship'] = $ship;

        $pieceList = array();
        if (isset($post['Package'])&& is_array($post['Package']) && count($post['Package']) > 0) {
            foreach ($post['Package'] as $package) {
                $_package = array();
                if (isset($package['packageType'])) {
                    if ($package['packageType'] == 'ENVELOPE') {
                        $_package['type'] = $package['packageType'];
                        if (isset($package['quantity'])) {
                            $_package['quantity'] = $package['quantity'];
                        }
                    } else {
                        $_package['type'] = $package['packageType'];
                        if (isset($package['weight'])) {
                            $_package['weight'] = $package['weight'];
                        }
                        if (isset($package['width'])) {
                            $_package['width'] = $package['width'];
                        }
                        if (isset($package['height'])) {
                            $_package['height'] = $package['height'];
                        }
                        if (isset($package['length'])) {
                            $_package['length'] = $package['length'];
                        }
                        if (isset($package['quantity'])) {
                            $_package['quantity'] = $package['quantity'];
                        }
                        if (isset($package['nonStandard'])) {
                            $_package['nonStandard'] = $package['nonStandard'];
                        }
                    }
                }
                if (count($_package) > 0) {
                    $pieceList[] = $_package;
                }
            }
        }
        $shipment['pieceList'] = $pieceList;

        $params = array(
            "authData" => array('username' => Mage::getStoreConfig('dhl24pl/webapi/wlogin',0), 'password' => Mage::getStoreConfig('dhl24pl/webapi/wpassword',0)),
        );
        $params['shipment'] = $shipment;

$context = stream_context_create(array('http' => array('header' => 'DHLApiOrgin: MAGENTO')));
        $_instance = new SoapClient(
            DHL_WEBAPI2,
            array( 'keep-alive' => true, 'connection_timeout' => 3, 'stream_context' => $context )
        );
        if( !$_instance ) {
            throw new CException( 'Brak połączenia z usługą' );
        }
        $result = $_instance->createShipment( $params );

        return $result;
    }

    /** Metoda tworzy tablice parametrow i tworzy przesylke za pomoca api servicepoint
     * @param $post - dane z formularza
     * @param $magentoData - dane pobrane z dhl
     * @param $magentoSender - dane nadawcy z panelu Magento
     * @return mixed
     * @throws CException
     */
    private function createServicePointShipment($post, $magentoData, $magentoSender) {

        $shipment = array();
        $shipmentData = array();

        $ship = array();
        //START::DANE NADAWCY
        $shipper = array();
        $shipperPreaviso = array();
        $shipperContact = array();
        if (isset($post['personName'])) {
            $shipperPreaviso['personName'] = $post['personName'];
            $shipperContact['personName'] = $post['personName'];

        }
        if (isset($post['phoneNumber'])) {
            $shipperPreaviso['phoneNumber'] = $post['phoneNumber'];
            $shipperContact['phoneNumber'] = $post['phoneNumber'];

        }
        if (isset($post['emailAddress'])) {
            $shipperPreaviso['emailAddress'] = $post['emailAddress'];
            $shipperContact['emailAddress'] = $post['emailAddress'];
        }
        $shipper['preaviso'] = $shipperPreaviso;
        $shipper['contact'] = $shipperContact;
        $shipperAddress = array();
        if (isset($post['payer'])) {
            if (($post['payer'] == 'N' || $post['payer'] == 'O')) {
                if (isset($post['name'])) {
                    $shipperAddress['name'] = $post['name'];
                }
            } else {
                if (isset($post['name2'])) {
                    $shipperAddress['name'] = $post['name2'];
                }
            }
        }
        if (isset($post['postcode'])) {
            $shipperAddress['postcode'] = $this->cleanPostalCode($post['postcode']);
        }
        if (isset($post['city'])) {
            $shipperAddress['city'] = $post['city'];
        }
        if (isset($post['street'])) {
            $shipperAddress['street'] = $post['street'];
        }
        if (isset($post['houseNumber'])) {
            $shipperAddress['houseNumber'] = $post['houseNumber'];
        }
        if (isset($post['apartmentNumber'])) {
            $shipperAddress['apartmentNumber'] = $post['apartmentNumber'];
        }
        $shipper['address'] = $shipperAddress;
        $ship['shipper'] = $shipper;
        //END::DANE NADAWCY
        //START::DANE ODBIORCY
        $receiver = array();
        $receiverPreaviso = array();
        $receiverContact = array();
        if (isset($post['rec_personName'])) {
            $receiverPreaviso['personName'] = $post['rec_personName'];
            $receiverContact['personName'] = $post['rec_personName'];

        }
        if (isset($post['rec_phoneNumber'])) {
            $receiverPreaviso['phoneNumber'] = $post['rec_phoneNumber'];
            $receiverContact['phoneNumber'] = $post['rec_phoneNumber'];

        }
        if (isset($post['rec_emailAddress'])) {
            $receiverPreaviso['emailAddress'] = $post['rec_emailAddress'];
            $receiverContact['emailAddress'] = $post['rec_emailAddress'];
        }
        $receiver['preaviso'] = $receiverPreaviso;
        $receiver['contact'] = $receiverContact;
        $receiverAddress = array();

        if (isset($post['rec_name'])) {
            $receiverAddress['name'] = $post['rec_name'];
        }
        if (isset($post['rec_postcode'])) {
            $receiverAddress['postcode'] = $this->cleanPostalCode($post['rec_postcode']);
        }
        if (isset($post['rec_city'])) {
            $receiverAddress['city'] = $post['rec_city'];
        }
        if (isset($post['rec_street'])) {
            $receiverAddress['street'] = $post['rec_street'];
        }
        if (isset($post['rec_houseNumber'])) {
            $receiverAddress['houseNumber'] = $post['rec_houseNumber'];
        }
        if (isset($post['rec_apartmentNumber'])) {
            $receiverAddress['apartmentNumber'] = $post['rec_apartmentNumber'];
        }
        $receiver['address'] = $receiverAddress;
        $ship['receiver'] = $receiver;
        //END::DANE ODBIORCY
        if (isset($post['lm_sap'])) {
            $ship['servicePointAccountNumber'] = $post['lm_sap'];
        }
        $shipmentData['ship'] = $ship;
        //START:: shipmentInfo
        $shipmentInfo = array();
        $shipmentInfo['dropOffType'] = 'REGULAR_PICKUP';
        $shipmentInfo['serviceType'] = 'LM';

        $billing = array();
        if (isset($post['payer'])) {
            if ($post['payer'] == 'N') {
                $billing['shippingPaymentType'] = 'SHIPPER';
                $billing['paymentType'] = 'BANK_TRANSFER';
            } elseif ($post['payer'] == 'O') {
                $billing['shippingPaymentType'] = 'RECEIVER';
                $billing['paymentType'] = 'CASH';
            } elseif($post['payer'] == 'T') {
                $billing['shippingPaymentType'] = 'USER';
                $billing['paymentType'] = 'BANK_TRANSFER';
            }
        }
        if (isset($magentoSender['sap'])) {
            $billing['billingAccountNumber'] = $magentoSender['sap'];
        }
        if (isset($post['costsCenter'])) {
            $billing['costsCenter'] = $post['costsCenter'];
        }
        $shipmentInfo['billing'] = $billing;

        $specialServices = array();
        if (isset($post['insurance']) && $post['insurance'] == '1') {
            $item = array();
            $item['serviceType'] = 'UBEZP';
            if (isset($post['insuranceValue'])) {
                $item['serviceValue'] = $post['insuranceValue'];
            }
            $specialServices[] = $item;
        }
        if (isset($post['collectOnDelivery']) && $post['collectOnDelivery'] == '1') {
            $item = array();
            $item['serviceType'] = 'COD';
            if (isset($post['collectOnDeliveryValue'])) {
                $item['serviceValue'] = $post['collectOnDeliveryValue'];
            }
            $item['collectOnDeliveryForm'] = 'BANK_TRANSFER';
            $specialServices[] = $item;
        }
        if (isset($post['predeliveryInformation']) && $post['predeliveryInformation'] == '1') {
            $item = array();
            $item['serviceType'] = 'PDI';
            $specialServices[] = $item;
        }
        if (isset($post['returnOnDelivery']) && $post['returnOnDelivery'] == '1') {
            $item = array();
            $item['serviceType'] = 'ROD';
            $specialServices[] = $item;
        }
        if (isset($post['proofOfDelivery']) && $post['proofOfDelivery'] == '1') {
            $item = array();
            $item['serviceType'] = 'POD';
            $specialServices[] = $item;
        }
        if (isset($post['deliveryEvening']) && $post['deliveryEvening'] == '1') {
            $item = array();
            $item['serviceType'] = '1722';
            $specialServices[] = $item;
        }
        if (isset($post['deliveryOnSaturday']) && $post['deliveryOnSaturday'] == '1') {
            $item = array();
            $item['serviceType'] = 'SOBOTA';
            $specialServices[] = $item;
        }
        if (isset($post['pickupOnSaturday']) && $post['pickupOnSaturday'] == '1') {
            $item = array();
            $item['serviceType'] = 'NAD_SOBOTA';
            $specialServices[] = $item;
        }
        if (isset($post['selfCollect']) && $post['selfCollect'] == '1') {
            $item = array();
            $item['serviceType'] = 'ODB';
            $specialServices[] = $item;
        }
        if (isset($post['deliveryToNeighbour']) && $post['deliveryToNeighbour'] == '1') {
            $item = array();
            $item['serviceType'] = 'SAS';
            $specialServices[] = $item;
        }
        $shipmentInfo['specialServices'] = $specialServices;



//        $specialServices = array();
//        if (isset($post['insurance']) && $post['insurance'] == '1') {
//            $item = array();
//            $item['serviceType'] = 'UBEZP';
//            if (isset($post['insuranceValue'])) {
//                $item['serviceValue'] = $post['insuranceValue'];
//            }
//            $specialServices[] = $item;
//        }
//        if (isset($post['returnOnDelivery']) && $post['returnOnDelivery'] == '1') {
//            $item = array();
//            $item['serviceType'] = 'ROD';
//            $specialServices[] = $item;
//        }
//        $shipmentInfo['specialServices'] = $specialServices;

        if (isset($post['shipmentDate'])) {
            $shipmentInfo['shipmentDate'] = $post['shipmentDate'];
        }
        $shipmentInfo['labelType'] = $magentoData['labelType'];
        $shipmentData['shipmentInfo'] = $shipmentInfo;
        //END:: shipmentInfo

        $pieceList = array();
        if (isset($post['Package'])&& is_array($post['Package']) && count($post['Package']) > 0) {
            foreach ($post['Package'] as $package) {
                $_package = array();
                if (isset($package['packageType'])) {
                    if ($package['packageType'] == 'ENVELOPE') {
                        $_package['type'] = $package['packageType'];
                        if (isset($package['quantity'])) {
                            $_package['quantity'] = $package['quantity'];
                        }
                    } else {
                        $_package['type'] = $package['packageType'];
                        if (isset($package['weight'])) {
                            $_package['weight'] = $package['weight'];
                        }
                        if (isset($package['width'])) {
                            $_package['width'] = $package['width'];
                        }
                        if (isset($package['height'])) {
                            $_package['height'] = $package['height'];
                        }
                        if (isset($package['length'])) {
                            $_package['lenght'] = $package['length'];
                        }
                        if (isset($package['quantity'])) {
                            $_package['quantity'] = $package['quantity'];
                        }
                        if (isset($package['nonStandard'])) {
                            $_package['nonStandard'] = $package['nonStandard'];
                        }
                    }
                }
                if (count($_package) > 0) {
                    $pieceList[] = $_package;
                }
            }
        }
        $shipmentData['pieceList'] = $pieceList;
        if (isset($post['content'])) {
            $shipmentData['content'] = $post['content'];
        }
        if (isset($post['comment'])) {
            $shipmentData['comment'] = $post['comment'];
        }
        if (isset($post['reference'])) {
            $shipmentData['reference'] = $post['reference'];
        }
        $shipment['authData'] = array('username' => Mage::getStoreConfig('dhl24pl/servicepoint/slogin',0), 'password' => Mage::getStoreConfig('dhl24pl/servicepoint/spassword',0));
        $shipment['shipmentData'] = $shipmentData;

        $params['shipment'] = $shipment;
		$context = stream_context_create(array('http' => array('header' => 'DHLApiOrgin: MAGENTO')));
        $_instance = new SoapClient(
            DHL_SERVICEPOINTAPI,
            array( 'keep-alive' => true, 'connection_timeout' => 3, 'stream_context' => $context )
        );
        if( !$_instance ) {
            throw new CException( 'Brak połączenia z usługą' );
        }
        $result = $_instance->createShipment( $params );

        return $result;
    }

    /** Metoda pobierajaca dostępne czasy przyjazdu kuriera
     * @param $date data zamawiania kuriera
     * @param $code - kod pocztowy
     * @return string komunikat z mozliwymi godzinami
     * @throws CException
     */
    private function getPostalCodeServices($date, $code) {

        $_instance = new SoapClient(
            DHL_WEBAPI2,
            array( 'keep-alive' => true, 'connection_timeout' => 3 )
        );
        if( !$_instance ) {
            throw new CException( 'Brak połączenia z usługą' );
        }
        $params = array(
            "authData" => array('username' => Mage::getStoreConfig('dhl24pl/webapi/wlogin',0), 'password' => Mage::getStoreConfig('dhl24pl/webapi/wpassword',0)),
            'postCode' => str_replace('-', '', $code),
            'pickupDate' => $date
        );
        $result = $_instance->getPostalCodeServices( $params );

        if ($result->getPostalCodeServicesResult->exPickupFrom == 'brak' || $result->getPostalCodeServicesResult->exPickupTo == 'brak') {
            $msg = 'Brak dostępnych godzin przyjazdu kuriera dla przesyłek ekspresowych.';
        } else {
            $msg = 'Możliwe godziny przyjazdu kuriera dla przesyłek ekspresowych: ' . $result->getPostalCodeServicesResult->exPickupFrom . ' - ' . $result->getPostalCodeServicesResult->exPickupTo . '.';
        }
        if ($result->getPostalCodeServicesResult->drPickupFrom == 'brak' || $result->getPostalCodeServicesResult->drPickupTo == 'brak') {
            $msg .= '<br />Brak dostępnych godzin przyjazdu kuriera dla przesyłek drobnicowych';
        } else {
            $msg .= '<br />Możliwe godziny przyjazdu kuriera dla przesyłek drobnicowych: ' . $result->getPostalCodeServicesResult->drPickupFrom . ' - ' . $result->getPostalCodeServicesResult->drPickupTo;
        }
        return $msg;
    }

    /***** METODY POMOCNICZE *****/

    /** Metoda sprawdza czy wybralismy opcje doreczenia do parceslhop, wywolywana jest api servicepoint
     * @param $allShipment - dane przesylki
     * @return bool
     */
    private function isLmShipment($allShipment) {
        if (isset($allShipment['deliveryToLM']) && $allShipment['deliveryToLM']) {
            return true;
        } else {
            return false;
        }
    }
    /** Metoda sprawdza czy dla zdefiniowanych danych mozna zamowic kuriera
     * @param $magentoData - dane z panelu magento z dhl
     * @param $shipment - nazwa wybranej przesylki, w przesylce jest tez informacja o dostepnosci kuriera
     * @return bool
     */
    private function canCourier($magentoData, $shipment) {
        if ((isset($magentoData['dropOffType']) && $magentoData['dropOffType'] == 'REQUEST_COURIER') || (isset($magentoData['shipments'][$shipment]) && isset($magentoData['shipments'][$shipment]['courier']) && $magentoData['shipments'][$shipment]['courier'])) {
            return true;
        } else {
            return false;
        }
    }

    /** Metoda sprawdza czy przesylka juz zostala utworzona dla danego zamowienia
     * @param $oid numer zamowienia z magento
     * @return bool
     */
    private function isAlreadyCreated($oid) {
        $model = Mage::getModel('sales/order')->loadByIncrementId($oid);
        $settings = $model->getDhl24plSettings();
        if (empty($settings)) {
            return false;
        } else {
            return true;
        }
    }

    /** Metoda pobiera zamowienie z magenyo
     * @param $oid numer zamowienia z magento
     * @return bool
     */
    private function getOrder($oid) {
        $model = Mage::getModel('sales/order/grid')->loadByIncrementId($oid);
        if ($model->getId()) {
            return $model;
        } else {
            return false;
        }
    }

    /** Metoda sprawdza czy istnieje zamowienie o podanym id
     * @param $oid numer zamowienia
     * @return bool
     */
    private function isOrder($oid) {
        $model = Mage::getModel('sales/order')->loadByIncrementId($oid);
        if ($model->getId()) {
            return true;
        } else {
            return false;
        }
    }

    /** Metoda na podstawie danych konfiguracyjnych sprawdza czy jest dostep do LM
     *
     */
    private function isLm() {
        $login = Mage::getStoreConfig('dhl24pl/servicepoint/slogin',0);
        $pass = Mage::getStoreConfig('dhl24pl/servicepoint/spassword',0);
        if (empty($login) || empty($pass)) {
            return false;
        } else {
            return true;
        }
    }

    private function cleanPostalCode($code) {
        return str_replace('-', '', $code);
    }

    /** Metoda pobiera dane odbiorcy z zamowienia
     * @param $orderId numer zamowienia w magento
     * @return array
     */
    private function getFromOrderData($orderId) {
        $model = Mage::getModel('sales/order')->loadByIncrementId($orderId);

        $data = array();
        $data['rec_name'] = $model->getShippingAddress()->getCompany() . ' ' . $model->getShippingAddress()->getFirstname().' '.$model->getShippingAddress()->getLastname();
        $data['rec_name'] = trim( $data['rec_name'] );
        $data['rec_name'] = substr( $data['rec_name'], 0, 60 );
        $data['rec_city'] = $model->getShippingAddress()->getCity();
        $data['rec_postcode'] = $model->getShippingAddress()->getPostcode();
        $data['rec_street'] = $model->getShippingAddress()->getStreetFull();
        $streetFull = $model->getShippingAddress()->getStreetFull();

        if ($streetArray = $this->preg($streetFull)) {
            if (isset($streetArray['street'])) {
                $data['rec_street'] = $streetArray['street'];
            }
            if (isset($streetArray['houseNumber'])) {
                $data['rec_houseNumber'] = $streetArray['houseNumber'];
            }
            if (isset($streetArray['apartmentNumber'])) {
                $data['rec_apartmentNumber'] = $streetArray['apartmentNumber'];
            }
        } else {
            $data['rec_houseNumber'] = '.';
        }
        $data['rec_phoneNumber'] = $model->getShippingAddress()->getTelephone();
        $data['rec_emailAddress'] = $model->getCustomerEmail();

        $neighbour = $model->getDhl24plNeighbor();
        if (!empty($neighbour)) {
            $neighbour = json_decode($neighbour);
            if (!empty($neighbour->neighbour_postcode) && !empty($neighbour->neighbour_city) && !empty($neighbour->neighbour_city)) {
                $data['neighbour_name'] = $neighbour->neighbour_name;
                $data['neighbour_postcode'] = $neighbour->neighbour_postcode;
                $data['neighbour_city'] = $neighbour->neighbour_city;
                $data['neighbour_street'] = $neighbour->neighbour_street;
                $data['neighbour_houseNumber'] = $neighbour->neighbour_houseNumber;
                $data['neighbour_apartmentNumber'] = $neighbour->neighbour_apartmentNumber;
                $data['neighbour_phoneNumber'] = $neighbour->neighbour_phoneNumber;
                $data['neighbour_emailAddress'] = $neighbour->neighbour_emailAddress;
                $data['deliveryToNeighbour'] = 1;
            }

        }
        $parcelshop = $model->getDhl24plParcelshop();
        if (!empty($parcelshop)) {
            $parcelshop = json_decode($parcelshop);
            $data['lm_sap'] = $parcelshop->sap;
            $data['deliveryToLM'] = 1;
        }
        //sprawdzenie czy została wybrana opcja cashondelivery
        $type = Mage::getStoreConfig('dhl24pl/cod/codtype','');
        $variante = Mage::getStoreConfig('dhl24pl/cod/codvariante','');
        $payment = $model->getPayment();
        $paymentType = $payment->getMethod();
        $price = false;
        if ($type == 'cashondelivery' || $type == 'phoenix_cashondelivery') {//jest wlaczona opcja COD
            if ($type == $paymentType) {//platnosc zgadza sie z typem
                if ($variante == 'all') {
                    $price = $model->getGrandTotal();
                } elseif ($variante == 'products') {
                    $price = $model->getSubtotalInclTax();
                }
            }
        }
        if ($price !== false) {//jest COD
            $data['insurance'] = 1;
            $data['insuranceValue'] = number_format($price, 2, '.', '');
            $data['collectOnDelivery'] = 1;
            $data['collectOnDeliveryValue'] = number_format($price, 2, '.', '');
        }

        return $data;
    }



    /** Metoda pobera dane na temat domyslnych ustawien magento z dhl24
     * dane są cachowane
     * @return array
     * @throws CException
     */
    private function getMagentoFromDHL() {
        try {
            $cache = Mage::app()->getCache();
            $magentoData = $cache->load("magentoData");
            if ($magentoData !== false) {
                return array('error' => false, 'data' => unserialize($magentoData));
            }
            $_instance = new SoapClient(
                DHL_MAGENTOAPI,
                array( 'keep-alive' => true, 'connection_timeout' => 3 )
            );
            if( !$_instance ) {
                throw new CException( 'Brak połączenia z usługą' );
            }
            $params = array(
                "authData" => array('username' => Mage::getStoreConfig('dhl24pl/webapi/wlogin',0), 'password' => Mage::getStoreConfig('dhl24pl/webapi/wpassword',0)),
            );
            $result = $_instance->getMagentoData( $params );

            $data = $this->prepareData($result->getMagentoDataResult);
            $cache->save(serialize($data), "magentoData", array(), 60 * 60 * 24);
            return array('error' => false, 'data' => $data);
        } catch (Exception $e) {
            return array('error' => $e->getMessage(), 'data' => '');
        }


    }

    /** Metoda przetwarza 'surowe' dane z panelu magento w przystepną tablicę
     * @param $result
     * @return array
     */
    private function prepareData($result) {
        $data = array();
        $data['labelType'] = $result->labelType;
        $data['dropOffType'] = $result->dropOffType;
        $data['sas'] = $result->sas;
        $data['ek'] = $result->ek;
        $senders = array();
        if (isset($result->senders->item)) {
            if (is_array($result->senders->item)) {
                foreach ($result->senders->item as $key => $value) {
                    $sender = array();
                    $sender['name'] = $value->name;
                    $sender['sap'] = $value->sap;
                    $sender['personName'] = $value->personName;
                    $sender['phoneNumber'] = $value->phoneNumber;
                    $sender['emailAddress'] = $value->emailAddress;
                    $sender['postcode'] = $value->postcode;
                    $sender['city'] = $value->city;
                    $sender['street'] = $value->street;
                    $sender['houseNumber'] = $value->houseNumber;
                    $sender['apartmentNumber'] = $value->apartmentNumber;
                    $senders[$value->name] = $sender;
                }
            } else if(is_object($result->senders->item)) {
                $value = $result->senders->item;
                $sender = array();
                $sender['name'] = $value->name;
                $sender['sap'] = $value->sap;
                $sender['personName'] = $value->personName;
                $sender['phoneNumber'] = $value->phoneNumber;
                $sender['emailAddress'] = $value->emailAddress;
                $sender['postcode'] = $value->postcode;
                $sender['city'] = $value->city;
                $sender['street'] = $value->street;
                $sender['houseNumber'] = $value->houseNumber;
                $sender['apartmentNumber'] = $value->apartmentNumber;
                $senders[$value->name] = $sender;
            }
        }
        $data['senders'] = $senders;

        $shipments = array();
        if (isset($result->shipments->item)) {
            if (is_array($result->shipments->item)) {
                foreach ($result->shipments->item as $key => $value) {
                    $shipment = array();
                    $shipment['name'] = $value->name;
                    $shipment['senderName'] = $value->senderName;
                    $shipment['default'] = $value->default;
                    $shipment['courier'] = $value->courier;
                    $shipment['product'] = $value->product;
                    $shipment['payer'] = $value->payer;
                    $shipment['packageType'] = $value->packageType;
                    $shipment['width'] = $value->width;
                    $shipment['height'] = $value->height;
                    $shipment['length'] = $value->length;
                    $shipment['weight'] = $value->weight;
                    $shipment['quantity'] = $value->quantity;
                    $shipment['nonStandard'] = $value->nonStandard;
                    $shipment['deliveryEvening'] = $value->deliveryEvening;
                    $shipment['deliveryOnSaturday'] = $value->deliveryOnSaturday;
                    $shipment['pickupOnSaturday'] = $value->pickupOnSaturday;
                    $shipment['collectOnDelivery'] = $value->collectOnDelivery;
                    $shipment['collectOnDeliveryValue'] = $value->collectOnDeliveryValue;
                    $shipment['insurance'] = $value->insurance;
                    $shipment['insuranceValue'] = $value->insuranceValue;
                    $shipment['returnOnDelivery'] = $value->returnOnDelivery;
                    $shipment['proofOfDelivery'] = $value->proofOfDelivery;
                    $shipment['selfCollect'] = $value->selfCollect;
                    $shipment['predeliveryInformation'] = $value->predeliveryInformation;
                    $shipment['deliveryToNeighbour'] = $value->deliveryToNeighbour;
                    $shipment['deliveryToLM'] = $value->deliveryToLM;
                    $shipment['content'] = $value->content;
                    $shipment['comment'] = $value->comment;
                    $shipment['costsCenter'] = $value->costsCenter;
                    $shipments[$value->name] = $shipment;
                }
            } else if(is_object($result->shipments->item)){
                $value = $result->shipments->item;
                $shipment = array();
                $shipment['name'] = $value->name;
                $shipment['senderName'] = $value->senderName;
                $shipment['default'] = $value->default;
                $shipment['courier'] = $value->courier;
                $shipment['product'] = $value->product;
                $shipment['payer'] = $value->payer;
                $shipment['packageType'] = $value->packageType;
                $shipment['width'] = $value->width;
                $shipment['height'] = $value->height;
                $shipment['length'] = $value->length;
                $shipment['weight'] = $value->weight;
                $shipment['quantity'] = $value->quantity;
                $shipment['nonStandard'] = $value->nonStandard;
                $shipment['deliveryEvening'] = $value->deliveryEvening;
                $shipment['deliveryOnSaturday'] = $value->deliveryOnSaturday;
                $shipment['pickupOnSaturday'] = $value->pickupOnSaturday;
                $shipment['collectOnDelivery'] = $value->collectOnDelivery;
                $shipment['collectOnDeliveryValue'] = $value->collectOnDeliveryValue;
                $shipment['insurance'] = $value->insurance;
                $shipment['insuranceValue'] = $value->insuranceValue;
                $shipment['returnOnDelivery'] = $value->returnOnDelivery;
                $shipment['proofOfDelivery'] = $value->proofOfDelivery;
                $shipment['selfCollect'] = $value->selfCollect;
                $shipment['predeliveryInformation'] = $value->predeliveryInformation;
                $shipment['deliveryToNeighbour'] = $value->deliveryToNeighbour;
                $shipment['deliveryToLM'] = $value->deliveryToLM;
                $shipment['content'] = $value->content;
                $shipment['comment'] = $value->comment;
                $shipment['costsCenter'] = $value->costsCenter;
                $shipments[$value->name] = $shipment;
            }
        }
        $data['shipments'] = $shipments;
        return $data;
    }

	//OLD PLUGIN -------------------------------------------------------------------------------------------------------
	private function renderParamList($params) 
	{
        $paramList = '?';
        foreach ($params as $name => $value) {
            $paramList .= '&' . $name . '=' . $value;
        }

        return $paramList;
    }
    
	public function processAction() 
	{
		$this->oid = $this->getRequest()->getParam('oid');
		$this->pid = $this->getRequest()->getParam('pid');
		$this->section = $this->getRequest()->getParam('section');
		
    	if ($this->login() != FAILURE) {
			$this->_redirectUrl(DHL_PROTOCOL . DHL_DOMAIN . '/uzytkownik/redirectEstoreUser' . $this->getUrlParams());
			return;
		}
		
		if (!empty($this->oid)) {
			$plainData = $this->getOrderData($this->oid);
			$serialized = serialize($plainData);
			$params = '?estore_login=true&estore_data=' . $serialized;
		}
		
		$this->_redirectUrl(DHL_PROTOCOL . DHL_DOMAIN . '/uzytkownik/zaloguj' . $params);
		
		return;
    }
    
	private function getUrlParams() 
	{
		if (in_array($this->getRequest()->getParam('section'), array('details', 'print', 'shipping')))
			$this->section = $this->getRequest()->getParam('section');
		else return;
		
        switch ($this->section) {
            case 'details':
                $params = array('param' => 'id', 'pid' => $this->pid, 'url' => 'przesylka/szczegoly');
                break;
            case 'print':
                $params = array('param' => 'id', 'pid' => $this->pid, 'url' => 'przesylka/drukuj');
                break;
            case 'shipping':
                $params = array('param' => 'null', 'pid' => 'null', 'url' => 'przesylka/utworz');
                break;

            default: $params = array();
        }

        $params = array_merge(array('sid' => $this->sid, 'section' => $this->section), $params);

        return $this->renderParamList($params);
    }
    
    public function getOrderData($orderId)
    {
    	/* @var $model Mage_Sales_Model_Order */
		$model = Mage::getModel('sales/order')->loadByIncrementId($orderId);
		// TODO if (empty($model->isObjectNew())
		
		$data = array();
		$data['ORDER_ID'] = $model->getIncrementId();
		$data['ODB_NAZWA'] = $model->getShippingAddress()->getCompany() . ' ' . $model->getShippingAddress()->getFirstname().' '.$model->getShippingAddress()->getLastname();
		$data['ODB_NAZWA'] = trim( $data['ODB_NAZWA'] );
        $data['ODB_NAZWA'] = substr( $data['ODB_NAZWA'], 0, 60 );
		$data['ODB_KRAJ'] = $model->getShippingAddress()->getCountryId();
		$data['ODB_MIEJSCOWOSC'] = $model->getShippingAddress()->getCity();
		$data['ODB_KOD_POCZTOWY'] = $model->getShippingAddress()->getPostcode();
		$data['ODB_ULICA'] = $model->getShippingAddress()->getStreetFull();
		$data['ODB_TELEFON'] = $model->getShippingAddress()->getTelephone();
		$data['ODB_EMAIL'] = $model->getCustomerEmail();
		$data['STORE_BACK_URL'] = Mage::helper("adminhtml")->getUrl('adminhtml/sales_order/index');
		$data['SAVE_URL'] = Mage::getUrl('dhl24pl/index/index');
		
		return $data;
    }
    
	public function login()
	{
		$url =  DHL_PROTOCOL . DHL_DOMAIN . '/uzytkownik/loginEstore';
		
		$data = array();
		
		//default settings for store_id=0
		$data[] = 'username=' . Mage::getStoreConfig('dhl24pl/settings/login',0);
		$data[] = 'password=' . md5(Mage::getStoreConfig('dhl24pl/settings/password',0));
		
		if (!empty($this->oid)) {
            $order = $this->getOrderData($this->oid);
            foreach ($order as $k => $v) {
               $data[] = "estore_data[" . $k . "]=" . $v;
            }
        }
		
		// print_r($data);
		
		$user_agent = "Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; DHL OLB Interface)";
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_POST, 1);
		
		if (count($data))
			curl_setopt($ch, CURLOPT_POSTFIELDS, join("&", $data));
		
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_USERAGENT, $user_agent);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		$output = curl_exec($ch);
		
		//print_r($output);
		//print_r(curl_error($ch));
		
		curl_close($ch);
		
		$this->sid = $output;
		
		return $output;
	}

    /** Metoda do rozbijania frazu z ulicą, nr domu i nr lokalu na ulicę nr domu i nr loklau jako tablice elemtnow
     * @param $streetFull
     * @return array|bool
     */
    public function preg($streetFull) {


        $data = array('street' => '', 'houseNumber' => '', 'apartmentNumber' => '');
        $streetFull = trim($streetFull);

        $pregLokal = 'l[\.]?|lo[\.]?|lok[\.]?|loka[\.]?|lokal[\.]?';
        $pregMieszkanie = 'm\.?|mi\.?|mie\.?|mies\.?|miesz\.?|mieszk\.?|mieszka\.?|mieszkan\.?|mieszkani\.?|mieszkanie\.?|mieszkania\.?';
        $pregLacznik = '(\/|\s|' . $pregLokal . '|' . $pregMieszkanie .'|przez){1}';
        $pregNrDomu = '(\d+[\/]?\d*[a-zA-Z]{0,3})';
        $pregNrMieszkania = '([a-zA-Z]{0,3}\d+[a-zA-Z]{0,3})';

        if ( preg_match('/^(.*)(?!(' . $pregNrDomu . '[\s]*' . $pregLacznik . '[\s]*' . $pregNrMieszkania . '))[\s]*' . $pregNrDomu . '[\s]*' . $pregLacznik . '[\s]*' . $pregNrMieszkania . '$/ ', $streetFull, $wynik) ) {
            if (isset($wynik[1])) {
                $data['street'] = $wynik[1];
            }
            if (isset($wynik[6])) {
                $data['houseNumber'] = $wynik[6];
            }
            if (isset($wynik[8])) {
                $data['apartmentNumber'] = $wynik[8];
            }
        } elseif (preg_match('/^([^\d]*[^\d\s])[\s]*' . $pregNrDomu . '[\s]*' . $pregLacznik . '[\s]*' . $pregNrMieszkania . '$/ ', $streetFull, $wynik)) {
            if (isset($wynik[1])) {
                $data['street'] = $wynik[1];
            }
            if (isset($wynik[2])) {
                $data['houseNumber'] = $wynik[2];
            }
            if (isset($wynik[4])) {
                $data['apartmentNumber'] = $wynik[4];
            }
        }else if (preg_match('/^(.*)(?!(' . $pregNrDomu . '))[\s]*' . $pregNrDomu . '$/ ', $streetFull, $wynik) ) {
            if (isset($wynik[1])) {
                $data['street'] = $wynik[1];
            }
            if (isset($wynik[4])) {
                $data['houseNumber'] = $wynik[4];
            }
        }  else if ( preg_match('/^([^\d]*[^\d\s])[\s]*(\d.*)$/ ', $streetFull, $wynik) ) {
            if (isset($wynik[1])) {
                $data['street'] = $wynik[1];
            }
            if (isset($wynik[2])) {
                $data['houseNumber'] = $wynik[2];
            }
        } else {
            $data = false;
        }
        return $data;
    }
}